using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

namespace P8_1
{
    /// <summary>
    /// Это главный тип игры
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        spriteComp gameObject;
        //Для запоминания позиции объекта
        Vector2 oldSprPos;
        //Для игрового объекта
        Texture2D texture;
        //Изображение стрелок
        Texture2D textureArrows;
        //Шаг перемещения спрайта, то есть - скорость
        float sprSpeed = 4;
        //Идентификатор касания
        int gameObjectId;
        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            graphics.IsFullScreen = true;
            // Частота кадра на Windows Phone по умолчанию — 30 кадров в секунду.
            TargetElapsedTime = TimeSpan.FromTicks(333333);
            // Дополнительный заряд аккумулятора заблокирован.
            InactiveSleepTime = TimeSpan.FromSeconds(1);
        }

        /// <summary>
        /// Позволяет игре выполнить инициализацию, необходимую перед запуском.
        /// Здесь можно запросить нужные службы и загрузить неграфический
        /// контент.  Вызов base.Initialize приведет к перебору всех компонентов и
        /// их инициализации.
        /// </summary>
        protected override void Initialize()
        {
            // ЗАДАЧА: добавьте здесь логику инициализации

            base.Initialize();
        }

        /// <summary>
        /// LoadContent будет вызываться в игре один раз; здесь загружается
        /// весь контент.
        /// </summary>
        protected override void LoadContent()
        {
            // Создайте новый SpriteBatch, который можно использовать для отрисовки текстур.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            Services.AddService(typeof(SpriteBatch), spriteBatch);
            texture = Content.Load<Texture2D>("BallandBats");
            CreateNewObject();
            textureArrows = Content.Load<Texture2D>("Arrows");
            // ЗАДАЧА: используйте здесь this.Content для загрузки контента игры
        }

        protected void CreateNewObject()
        {
            gameObject = new spriteComp(this, ref texture,
                new Rectangle(18, 9, 17, 88), new Vector2(100, 150));
            Components.Add(gameObject);
        }

        /// <summary>
        /// UnloadContent будет вызываться в игре один раз; здесь выгружается
        /// весь контент.
        /// </summary>
        protected override void UnloadContent()
        {
            // ЗАДАЧА: выгрузите здесь весь контент, не относящийся к ContentManager
        }
        //Проверка на столкновение
        //игрового объекта и прямоугольника
        //ограничивающего блок управления
        bool IsCollide(spriteComp sp1)
        {
            if (sp1.sprPos().X < 500 + 300 &&
                sp1.sprPos().X + sp1.sprRect().Width > 500 &&
                sp1.sprPos().Y < 280 + 200 &&
                sp1.sprPos().Y + sp1.sprRect().Height > 280)
            {
                return true;
            }
            else return false;

        }

        /// <summary>
        /// Позволяет игре запускать логику обновления мира,
        /// проверки столкновений, получения ввода и воспроизведения звуков.
        /// </summary>
        /// <param name="gameTime">Предоставляет моментальный снимок значений времени.</param>
        protected override void Update(GameTime gameTime)
        {
            // Позволяет выйти из игры
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();
            //Запоминаем текущую позицию игрового объекта
            oldSprPos = gameObject.sprPos();

            //Получаем коллекцию объектов, содержащих информацию о касаниях экрана
            TouchCollection touchLocations = TouchPanel.GetState();
            //Перебираем коллекцию, присваивая объекту координаты касания
            foreach (TouchLocation touchLocation in touchLocations)
            {
                if (touchLocation.State == TouchLocationState.Pressed)
                {
                    //Для объекта, раположенного справа
                    //Стрелка "Влево"
                    if (touchLocation.Position.X > 500 && touchLocation.Position.X < 600
                        && touchLocation.Position.Y > 380 && touchLocation.Position.Y < 480)
                    {
                        gameObject.Move(new Vector2(-sprSpeed, 0));
                        gameObjectId = touchLocation.Id;
                       
                    }
                    //Стрелка "Вправо"
                    if (touchLocation.Position.X > 700 && touchLocation.Position.X < 800
                        && touchLocation.Position.Y > 380 && touchLocation.Position.Y < 480)
                    {
                        gameObject.Move(new Vector2(sprSpeed, 0));
                        gameObjectId = touchLocation.Id;
                    }
                    //Стрелка "Вниз"
                    if (touchLocation.Position.X > 600 && touchLocation.Position.X < 700
                        && touchLocation.Position.Y > 380 && touchLocation.Position.Y < 480)
                    {
                        gameObject.Move(new Vector2(0, sprSpeed));
                        gameObjectId = touchLocation.Id;
                    }
                    //Стрелка "Вверх"
                    if (touchLocation.Position.X > 600 && touchLocation.Position.X < 700
                        && touchLocation.Position.Y > 280 && touchLocation.Position.Y < 380)
                    {
                        gameObject.Move(new Vector2(0, -sprSpeed));
                        gameObjectId = touchLocation.Id;
                    }
                    
                }
                //Если касание окончилось
                if (touchLocation.State == TouchLocationState.Released)
                {
                    if (touchLocation.Id == gameObjectId)
                    {
                        gameObject.Move(new Vector2(0, 0));
                    }
               
                }
            }


            base.Update(gameTime);
            //Если произошло столкновение - возвращаем игровой объект в предыдущую позицию
            if (IsCollide(gameObject))
            {
                gameObject.setSprPos(oldSprPos);
            }
        }

        /// <summary>
        /// Вызывается, когда игра отрисовывается.
        /// </summary>
        /// <param name="gameTime">Предоставляет моментальный снимок значений времени.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            spriteBatch.Begin();
            //Сначала выведем изображение стрелок для управления объектом
            spriteBatch.Draw(textureArrows, new Rectangle(500, 280, 300, 200), Color.White);
            //Игровой компонент будет выведен поверх изображения стрелок
            base.Draw(gameTime);
            spriteBatch.End();

        }
    }
}
